import androidx.compose.animation.AnimatedContent
import androidx.compose.animation.AnimatedVisibility
import androidx.compose.animation.core.Spring
import androidx.compose.animation.core.spring
import androidx.compose.animation.expandVertically
import androidx.compose.animation.fadeIn
import androidx.compose.animation.fadeOut
import androidx.compose.animation.slideInHorizontally
import androidx.compose.animation.slideOutHorizontally
import androidx.compose.animation.togetherWith
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.interaction.MutableInteractionSource
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.material.icons.Icons
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.material3.TextField
import androidx.compose.material3.TextFieldDefaults
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.derivedStateOf
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.BiasAlignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.focus.FocusRequester
import androidx.compose.ui.focus.focusRequester
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.painter.BitmapPainter
import androidx.compose.ui.input.key.Key
import androidx.compose.ui.input.key.KeyEventType
import androidx.compose.ui.input.key.key
import androidx.compose.ui.input.key.onKeyEvent
import androidx.compose.ui.input.key.type
import androidx.compose.ui.res.loadImageBitmap
import androidx.compose.ui.res.useResource
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.unit.DpSize
import androidx.compose.ui.unit.dp
import androidx.compose.ui.window.DialogWindow
import androidx.compose.ui.window.Window
import androidx.compose.ui.window.WindowPosition
import androidx.compose.ui.window.application
import androidx.compose.ui.window.rememberDialogState
import androidx.compose.ui.window.rememberWindowState
import notifications.NotificationCenter
import theme.Minimize
import theme.QuickLaunchTheme
import viewModel.QuickLaunchInputCoordinator
import viewModel.ShortCutHandler
import windows.ShortcutEditWindow


@Composable
fun App(
    minimize: () -> Unit,
    openShortcuts: () -> Unit
) {
    LaunchedEffect(Unit) {
        QuickLaunchInputCoordinator.uiEvents.collect { event ->
            when (event) {
                is QuickLaunchInputCoordinator.QLUiEvent.HelpEvent -> {

                }

                is QuickLaunchInputCoordinator.QLUiEvent.OpenShortcutWindowEvent -> {
                    openShortcuts()
                }
            }
        }
    }
    val notification by NotificationCenter.message.collectAsState("")
    Column(
        verticalArrangement = Arrangement.SpaceAround,
        modifier = Modifier.background(color = Color(0f, 0f, 0f, 0.8f))
    ) {
        Row(modifier = Modifier.fillMaxWidth()) {
            val text = remember { mutableStateOf("") }
            val focusRequester = remember { FocusRequester() }
            TextField(
                colors = TextFieldDefaults.colors(
                    focusedContainerColor = Color.Transparent,
                    unfocusedContainerColor = Color.Transparent,
                    focusedIndicatorColor = Color.Transparent,
                    unfocusedIndicatorColor = Color.Transparent
                ),
                textStyle = MaterialTheme.typography.displayMedium.copy(
                    fontWeight = FontWeight(200)
                ),
                modifier = Modifier.focusRequester(focusRequester).fillMaxHeight(0.7f).fillMaxWidth(0.925f).onKeyEvent {
                    if (it.type == KeyEventType.KeyUp && it.key == Key.Enter) {
                        QuickLaunchInputCoordinator.enter(text.value)
                        text.value = ""
                        return@onKeyEvent true
                    }
                    return@onKeyEvent false
                },
                value = text.value,
                onValueChange = {
                    text.value = it
                },
                maxLines = 1
            )
            LaunchedEffect(Unit) {
                focusRequester.requestFocus()
            }
            Box(
                modifier = Modifier
                    .padding(0.dp)
                    .background(Color.Transparent)
                    .width(30.dp)
                    .height(30.dp)
                    .clickable(
                        indication = null,
                        interactionSource = remember { MutableInteractionSource() }
                    ) { minimize() },
                contentAlignment = Alignment.Center
            ) {
                Icon(
                    imageVector = Icons.Default.Minimize,
                    "minimize",
                    tint = Color.White,
                    modifier = Modifier.fillMaxSize()
                )
            }
        }
        val expanded by remember { derivedStateOf { notification.isNotEmpty() } }
        AnimatedVisibility(
            visible = expanded,
            enter = expandVertically(
                spring(
                    stiffness = Spring.StiffnessLow,
                    dampingRatio = Spring.DampingRatioHighBouncy
                )
            )
        ) {
            AnimatedContent(
                targetState = notification,
                transitionSpec = {
                    fadeIn() + slideInHorizontally() togetherWith fadeOut() + slideOutHorizontally(
                        targetOffsetX = { it / 2 },
                    )
                }
            ) {
                Text(
                    it,
                    color = Color.Yellow,
                    modifier = Modifier.padding(start = 16.dp, bottom = 8.dp, top = 4.dp)
                )
            }


        }
    }

}


fun main() {
    application {
        val windowState =
            rememberWindowState(position = WindowPosition(BiasAlignment(0f, -0.4f)), size = DpSize(400.dp, 104.dp))

        QuickLaunchTheme {
            Window(
                icon = BitmapPainter(useResource("icon.png", ::loadImageBitmap)),
                transparent = true,
                undecorated = true,
                resizable = false,
                title = "QuickLaunch",
                state = windowState,
                onCloseRequest = ::exitApplication
            ) {
                val openHelpWindow = remember { mutableStateOf(false) }
                App(
                    minimize = { windowState.isMinimized = true },
                    openShortcuts = { openHelpWindow.value = true }
                )
                DialogWindow(
                    transparent = false,
                    visible = openHelpWindow.value,
                    title = "Edit shortcuts",
                    onCloseRequest = {
                        openHelpWindow.value = false
                        ShortCutHandler.persistToStorage()
                    },
                    state = rememberDialogState(
                        size = DpSize(800.dp, 600.dp),
                        position = WindowPosition(Alignment.Center)
                    )
                ) {
                    ShortcutEditWindow()
                }
            }
        }
    }
}

